package de.szut.lf8_starter.integration.project;

import de.szut.lf8_starter.project.ProjectEntity;
import de.szut.lf8_starter.project.ProjectRepository;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.*;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;

import java.time.LocalDate;
import java.util.List;
import java.util.Map;
import java.util.Objects;

import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@SpringBootTest
@AutoConfigureMockMvc(addFilters = false)
class AddEmployeeToProjectActionIntegrationTest {
    @Autowired
    private MockMvc mockMvc;

    @Autowired
    private RestTemplate restTemplate;

    @Autowired
    private ProjectRepository projectRepository;

    @Test
    void addEmployeeToProjectTest() throws Exception {
        ProjectEntity project = new ProjectEntity();
        project.setComment("comment");
        project.setContractor(1);
        project.setContractorName("contractorName");
        project.setEndDate(LocalDate.of(2024, 1, 1));
        project.setLeadingEmployee(1);
        project.setName("name");
        project.setStartDate(LocalDate.of(2021, 1, 1));
        project.setEmployees(List.of(1L, 2L, 3L));
        this.projectRepository.save(project);


        mockMvc.perform(post("/projects/{projectId}/employees/{employeeId}", 1, 312)
                        .header(HttpHeaders.AUTHORIZATION, getBearerToken())
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isNoContent());

        ProjectEntity updatedProject = projectRepository.findById(1L).get();

        assert updatedProject.getEmployees().contains(312L);
    }

    private String getBearerToken() {
        String url = "https://keycloak.szut.dev/auth/realms/szut/protocol/openid-connect/token";

        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_FORM_URLENCODED);

        MultiValueMap<String, String> map = new LinkedMultiValueMap<>();
        map.add("grant_type", "password");
        map.add("client_id", "employee-management-service");
        map.add("username", "user");
        map.add("password", "test");

        HttpEntity<MultiValueMap<String, String>> request = new HttpEntity<>(map, headers);

        ResponseEntity<Map> response = this.restTemplate.exchange(url, HttpMethod.POST, request, Map.class);

        return Objects.requireNonNull(response.getBody()).get("access_token").toString();
    }
}
